import { XhsClient } from '@kevisual/xhs/libs/xhs.ts';
import { Sequelize } from 'sequelize';
// import { createSequelize } from '@kevisual/xhs/services/xhs-db/db.ts';
import path from 'node:path';
import fs from 'node:fs';
export { XhsClient };
type XhsClientOptions = {
  key: string;
  cookie: string;
  signConfig?: {
    signUrl: string;
  };
  initDB?: boolean;
  [key: string]: any;
};
type XhsClientMap = {
  client: XhsClient;
  key: string;
  options: XhsClientOptions;
  db?: Sequelize;
};
type XhsServicesOptions = {
  root?: string;
};
/**
 * @description XhsServices is a singleton class that manages the XhsClient instances.
 * It is used to create and manage the XhsClient instances.
 */
export class XhsServices {
  map: Map<string, XhsClientMap> = new Map();
  root: string = 'root';
  constructor(opts?: XhsServicesOptions) {
    this.root = opts?.root || this.root;
  }
  createClient(options: XhsClientOptions) {
    const { key, cookie, signConfig } = options;
    if (this.map.has(key)) {
      return this.map.get(key);
    }
    const client = new XhsClient({ cookie });
    client.signConfig = signConfig;
    const storagePath = options.storage || `db-sqlite/xhs-${key}.db`;
    const storage = path.resolve(storagePath);
    const dir = path.dirname(storage);
    if (!fs.existsSync(dir)) {
      fs.mkdirSync(dir, { recursive: true });
    }
    let isNew = options?.initDB ?? false;
    if (!fs.existsSync(storage) || !isNew) {
      isNew = true;
    }
    // const db = createSequelize({ storage: storage });
    const xhsClientMap = {
      client,
      key,
      options,
      // db,
    };
    // if (isNew) {
    //   this.initDb(xhsClientMap);
    // }
    this.map.set(key, xhsClientMap);

    return client;
  }
  async initDb(xhsClientMap: XhsClientMap) {
    //
  }

  createRoot(options: Partial<XhsClientOptions>) {
    options.key = options.key || this.root;
    return this.createClient(options as XhsClientOptions) as XhsClient;
  }
  getKey(key?: string) {
    if (!key) key = this.root;
    return key;
  }
  /**
   * Get the XhsClient instance by key
   * @param key
   * @returns
   */
  getClient(key?: string) {
    const xhsClient = this.map.get(this.getKey(key));
    return xhsClient.client;
  }
  getXhsClient(key?: string) {
    const xhsClient = this.map.get(this.getKey(key));
    return xhsClient;
  }
}
